package Conifer::Plugin::DataHelpers;
use Mojo::Base 'Mojolicious::Plugin';
use 5.010001;
use utf8;
use List::Util 'first';
use Mojo::ByteStream 'b';
use Mojo::Util 'xml_escape';
use Time::HiRes 'time', 'gettimeofday';
use Data::Dumper::Simple;
use MIME::Base64;
use AnyEvent;

use constant DEBUG => $ENV{CONIFER_DATAHELPERS_DEBUG} || 0;


sub register {
	my ($self, $app) = @_;
	
	
	# Add "add_data" helper
	$app->helper(
		add_data => sub {
			my $c = shift;
			my $data = shift;
			
			my ($data_id, $hash) = $app->key_gen("data");
			
			#~ say "DataHelpers: add_data:\n\tDATA_ID: $data_id\n\tHASH: $hash\n\tDATA: ".Dumper($data);
			
			for my $key (keys %$data) {
				Conifer->redis->set("data:$data_id:$key" => $data->{$key});
				#~ say "DataHelpers: add_data:\n\tRedis_key: data:$data_id:$key\n\tVALUE: $data->{$key}";
			}
			
			return ( $data_id, b($hash) );
		}
	);
	
	
	# Add "get_data" helper
	$app->helper(
		get_data => sub {
			my $c = shift;
			my $hash = shift;
			my @keys = @_;
			#~ my $aecv = AE::cv;
			
			my $data_id = $app->key_lookup("data",$hash);
			
			my %data;
			#~ $aecv->begin (sub { shift->send(\%data) });	# Outer CV
			
			my $counter = scalar @keys;
			for my $key (@keys) {
				#~ $aecv->begin;	# Job CV
				#~ $data{$key} = Conifer->redis->get("data:$data_id:$key");
				Conifer->redis->get("data:$data_id:$key" => sub {
					my ($redis, $res) = @_;
					say "DataHelpers: get_data:\n\tRedis_key: data:$data_id:$key\n\tRES: ".Dumper($res) if DEBUG;
					$data{$key} = $res->[0];
					say "DataHelpers: get_data:\n\tRedis_key: data:$data_id:$key\n\tVALUE: $data{$key}" if DEBUG;
					#~ $aecv->end;
					$redis->stop;
				});
			}
			Conifer->redis->start;
			
			#~ $aecv->end;	# Outer CV
			#~ $aecv->recv;	# non-blocking wait for all jobs
			say "DataHelpers: get_data:\n\tDATA_ID: $data_id\n\tHASH: $hash\n\tDATA: ".Dumper(%data) if DEBUG;
			
			return ( $data_id, \%data );
		}
	);
	
	# Add "get_board_info" helper
	$app->helper(
		get_board_info => sub {
			my $c = shift;
			#~ my $hash = shift;
			my $data_id = shift;
			my @keys = @_;
			#~ my $aecv = AE::cv;
			
			#~ my $data_id = $app->key_lookup("data",$hash);
			
			my %data;
			#~ $aecv->begin (sub { shift->send(\%data) });	# Outer CV
			
			my $counter = scalar @keys;
			for my $key (@keys) {
				#~ $aecv->begin;	# Job CV
				#~ $data{$key} = Conifer->redis->get("data:$data_id:$key");
				Conifer->redis->get("board:$data_id:$key" => sub {
					my ($redis, $res) = @_;
					$data{$key} = $res->[0];
					say "DataHelpers: get_board_info:\n\tRedis_key: data:$data_id:$key\n\tVALUE: $data{$key}" if DEBUG;
					#~ $aecv->end;
					$redis->stop if --$counter == 0;
				});
			}
			Conifer->redis->start;
			
			#~ $aecv->end;	# Outer CV
			#~ $aecv->recv;	# non-blocking wait for all jobs
			say "DataHelpers: get_board_info:\n\tDATA_ID: $data_id\n\tDATA: ".Dumper(%data);
			
			#~ return ( $data_id, \%data );
			return ( \%data );
		}
	);
	
	# Add "get_list" helper
	$app->helper(
		get_list => sub {
			my $c = shift;
			my ($key, $start, $stop) = @_;
			$start //= 0;
			$stop //= -1;
			#~ my $t = Mojo::IOLoop->trigger;
						
			my $res;
			
			#~ $t->begin;
			Conifer->redis->zrange($key, $start, $stop, sub {
				my $redis = shift;
				$res = shift;
				
				say "DataHelpers: get_list:\n\tRedis_key: $key\n\tRES: ".Dumper($res) if DEBUG;
				$redis->stop;
				#~ $t->end(1,2,3);
			});
			Conifer->redis->start;
			
			#~ $t->start;
			#~ say "DataHelpers: get_board_list:\n\tRES: ".Dumper($res);
			
			return ( $res );
		}
	);
	
	
}

1;
