package Conifer::Plugin::Helpers;
use Mojo::Base 'Mojolicious::Plugin';
use 5.010001;
use utf8;
use List::Util 'first';
use Mojo::ByteStream 'b';
use Mojo::Util 'xml_escape';
use Time::HiRes 'time', 'gettimeofday';
use Data::Dumper::Simple;
use MIME::Base64;
use AnyEvent;

use constant DEBUG => $ENV{CONIFER_HELPERS_DEBUG} || 0;


sub register {
	my ($self, $app) = @_;
	
	
	# Add "key_gen" helper
	$app->helper(
		key_gen => sub {
			# new id, hash
			my $c = shift;
			my $namespace = shift;
			my $aecv = AE::cv;
			
			#~ my $counter = Conifer->redis->incr("$namespace:counter");
			my $counter;
			$aecv->begin (sub { shift->send($counter) });	# Outer CV
			
			
				$aecv->begin;	# Job CV
				#~ $data{$key} = Conifer->redis->get("data:$data_id:$key");
				Conifer->redis->incr("$namespace:counter" => sub {
					my ($redis, $res) = @_;
					say "Helpers: key_gen:\n\tRedis_key: $namespace:counter\n\tRES: ".Dumper($res) if DEBUG;
					$counter = $res->[0];
					say "Helpers: key_gen:\n\tRedis_key: $namespace:counter\n\tCOUNTER: $counter" if DEBUG;
					$aecv->end;
				});
			
			
			$aecv->end;	# Outer CV
			$aecv->recv;	# non-blocking wait for all jobs
			
			
			my $hash = encode_base64($counter);
			
			chomp $hash;
			$hash =~ s/=//g;
			
			say "Helpers: key_gen:\n\tNAMESPACE: $namespace\n\tCOUNTER: $counter\n\tHASH: $hash" if DEBUG;
			
			return( b($counter), b($hash));
		}
	);
	
	# Add "key_lookup" helper
	$app->helper(
		key_lookup => sub {
			# hash -> id
			my $c = shift;
			my ($namespace, $hash) = @_;
			my $id = decode_base64($hash);
			
			#~ print "Helpers: key_lookup:\n\tNAMESPACE: $namespace\n\tID: $id\n\tHASH: $hash\n";
			
			return b($id);
		}
	);
	
	# Add "time_and_rand" helper
	$app->helper(
		time_and_rand => sub {
			# Date and time
			my $c = shift;
			
			my ($sec, $min, $hour, $mday, $month, $year) = localtime;
			$month = $month + 1;
			$year = $year + 1900;
			
			my $microseconds;
			(undef, $microseconds) = gettimeofday;
			
			# Random number(0 ~ 99999)
			my $rand_num = int(rand 10);
			
			# Create file name form datatime and random number
			# (like image-20091014051023-78973)
			#~ my $name = sprintf("%04s%02s%02s%02s%02s%02s.%05s-%05s",
				#~ $year, $month, $mday, $hour, $min, $sec, $microseconds, $rand_num
			#~ );	
			# 
			my $name = sprintf(
				"%04s%02s%02s%02s%02s%02s.%05s-%01s",
				$year, $month, $mday, $hour, $min, $sec, $microseconds, $rand_num
			);
			
			return b($name);
		}
	);
	
}

1;
